package ch.zhaw.gartenverwaltung;

import ch.zhaw.gartenverwaltung.models.Garden;
import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.models.PlantNotFoundException;
import ch.zhaw.gartenverwaltung.types.GrowthPhaseType;
import ch.zhaw.gartenverwaltung.types.Plant;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.stage.Stage;
import javafx.util.Callback;

import java.io.IOException;
import java.net.URL;
import java.time.LocalDate;
import java.util.List;
import java.util.ResourceBundle;

public class SelectSowDayController implements Initializable {
    private Plant selectedPlant = null;
    private Garden garden;

    @SuppressWarnings("unused")
    public void injectDependencies(Garden garden) {
        this.garden = garden;
    }

    @FXML
    private DatePicker datepicker;

    @FXML
    private Label popup_label;

    @FXML
    private Button save_button;

    @FXML
    private RadioButton harvest_radio;

    /**
     * close the date selector window
     * @param event event
     */
    @FXML
    void cancel(ActionEvent event) {
        closeWindow();
    }

    /**
     * get sow date from datePicker or calculate sow date from harvest date
     * save selected plant and sow date
     * @param event event
     */
    @FXML
    void save(ActionEvent event) throws HardinessZoneNotSetException, IOException, PlantNotFoundException {
        LocalDate sowDate = datepicker.getValue();
        if (harvest_radio.isSelected()) {
            //ToDo method to get current lifecycle group in plant
            sowDate = selectedPlant.sowDateFromHarvestDate(datepicker.getValue(), 0);
        }
        garden.plantAsCrop(selectedPlant, sowDate);
        closeWindow();
    }

    /**
     * save the plant which will be planted and update label
     * @param plant Plant
     */
    public void setSelectedPlant(Plant plant) {
        selectedPlant = plant;
        popup_label.setText(String.format("Select Harvest/Sow Date for %s:", selectedPlant.name()));
    }

    /**
     * add listener and set default values
     * {@inheritDoc}
     * @param location location
     * @param resources resources
     */
    @Override
    public void initialize(URL location, ResourceBundle resources) {
        clearDatePickerEntries();

        Callback<DatePicker, DateCell> dayCellFactory= getDayCellFactory();
        datepicker.setDayCellFactory(dayCellFactory);
        datepicker.setEditable(false);

        save_button.disableProperty().bind(datepicker.valueProperty().isNull());
    }

    /**
     * clear date picker editor when radio button is changed
     */
    private void clearDatePickerEntries() {
        harvest_radio.selectedProperty().addListener((observable, oldValue, isNowSelected) -> datepicker.setValue(null));
    }

    /**
     * date picker disable/enable dates according to selected plant: sow or harvest day
     * @return cellFactory of datePicker
     */
    private Callback<DatePicker, DateCell> getDayCellFactory() {

        return (datePicker) -> new DateCell() {
            @Override
            public void updateItem(LocalDate item, boolean empty) {
                super.updateItem(item, empty);
                setDisable(true);
                setStyle("-fx-background-color: #ffc0cb;");
                List<LocalDate> dates;
                LocalDate today = LocalDate.now();
                if (harvest_radio.isSelected()) {
                    dates = selectedPlant.getDateListOfGrowthPhase(GrowthPhaseType.HARVEST);
                } else {
                    dates = selectedPlant.getDateListOfGrowthPhase(GrowthPhaseType.SOW);
                }
                for (LocalDate date : dates) {
                    if (item.getMonth() == date.getMonth()
                            && item.getDayOfMonth() == date.getDayOfMonth()
                            && item.compareTo(today) > 0) {
                        setDisable(false);
                        setStyle("-fx-background-color: #32CD32;");
                    }
                }
                if ((harvest_radio.isSelected() && selectedPlant.sowDateFromHarvestDate(item, 0).compareTo(today) < 0)) {
                    setDisable(true);
                    setStyle("-fx-background-color: #ffc0cb;");
                }
            }
        };
    }

    /**
     * close date picker window
     */
    private void closeWindow() {
        Stage stage = (Stage) save_button.getScene().getWindow();
        stage.close();
    }
}
